﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Security.Cryptography;
using System.IO.Packaging;
using System.IO;

namespace ZipPackageSample {
    public partial class MainForm : Form {

        private const string _fileName = @"Sample.zip";
        private static readonly Uri _textUri = new Uri("/Pero/Jozo/text.txt", UriKind.Relative);
        private static readonly Uri _pictureUri = new Uri("/picture.jpg", UriKind.Relative);
        private static readonly Uri _binaryUri = new Uri("/1gb_random.bin", UriKind.Relative);


        public MainForm() {
            InitializeComponent();
        }

        private void writeButton_Click(object sender, EventArgs e) {
            byte[] rndMB = new byte[1024 * 1024];
            RandomNumberGenerator.Create().GetBytes(rndMB);

            this.Enabled = false;
            using (var package = (ZipPackage)ZipPackage.Open(_fileName, FileMode.Create, FileAccess.ReadWrite)) {

                var part1 = package.CreatePart(_textUri, System.Net.Mime.MediaTypeNames.Text.Plain, CompressionOption.Maximum);
                using (var outputStream = part1.GetStream())
                using (var inputStream = System.Reflection.Assembly.GetEntryAssembly().GetManifestResourceStream("ZipPackageSample.Text.txt")) {
                    byte[] buffer = new byte[inputStream.Length];
                    inputStream.Read(buffer, 0, buffer.Length);
                    outputStream.Write(buffer, 0, buffer.Length);
                }

                var part2 = package.CreatePart(_pictureUri, System.Net.Mime.MediaTypeNames.Image.Jpeg, CompressionOption.NotCompressed);
                using (var outputStream = part2.GetStream())
                using (var inputStream = System.Reflection.Assembly.GetEntryAssembly().GetManifestResourceStream("ZipPackageSample.Picture.jpg")) {
                    byte[] buffer = new byte[inputStream.Length];
                    inputStream.Read(buffer, 0, buffer.Length);
                    outputStream.Write(buffer, 0, buffer.Length);
                }

                var part3 = package.CreatePart(_binaryUri, System.Net.Mime.MediaTypeNames.Application.Octet, CompressionOption.NotCompressed);
                using (var stream = part3.GetStream()) {
                    for (int i = 0; i < 1024; ++i) {
                        Trace.TraceInformation(i.ToString());
                        stream.Write(rndMB, 0, rndMB.Length);
                    }
                }

                package.Flush();
            }
            this.Enabled = true;

        }


        private void readButton_Click(object sender, EventArgs e) {
            this.Enabled = false;
            using (var package = (ZipPackage)ZipPackage.Open(_fileName, FileMode.Open, FileAccess.Read)) {
                var part1 = package.GetPart(_textUri);
                text.Text = new System.IO.StreamReader(part1.GetStream()).ReadToEnd();

                var part2 = package.GetPart(_pictureUri);
                picture.Image = Bitmap.FromStream(part2.GetStream());

                var part3 = package.GetPart(_binaryUri);
                byte[] rndMB = new byte[1024 * 1024];
                using (var inputStream = part3.GetStream()) {
                    while (inputStream.Read(rndMB, 0, rndMB.Length) > 0) {
                        Trace.TraceInformation((inputStream.Position / (1024 * 1024)).ToString());
                    }
                }
            }
            this.Enabled = true;
        }

    }
}
